// 
// このコードは、https://demo.mescius.jp/diodocs/pdf のサンプルの一部として提供されています。
// © MESCIUS inc. All rights reserved.
// 
using System;
using System.IO;
using System.Reflection;
using System.Collections.Generic;
using System.Diagnostics;
using GrapeCity.Documents.Pdf;
using GrapeCity.Documents.Pdf.Layers;
//
// このプログラムは、RotatedText クラスのインスタンスを作成し、それを使用して PDF ファイルを生成します。
// また、生成された PDF をマルチフレームの TIFF として保存し、PDF の最初のページを SVG として保存します
// （他に、JPEG、PNG、BMP、GIF 形式への保存にも対応しています）。
// このサンプルのビルドと実行に必要な DioDocs パッケージは NuGet.org から入手可能です。
//
// コマンドラインスイッチ：
// /y, -y ：すべてデフォルトとみなし、既存のファイルを上書きします。
// /q, -q ：エコーをオフにします（/y の指定がない場合は無視されます）。
// /f path, -f path ：ベースファイルのパス/名前として 'name' を使用します。
//
// プログラムの終了コード：
// 0 ：プログラムが正常に終了しました。
// 1 ：予期せぬエラーが発生しました。
// 2 ：プログラムがユーザにより中断されました。
//
var yes2all = false;
var quiet = false;
var norandom = false;
MemoryStream outMs = null;
TextWriter outTw = null;
try
{
    var fname = "RotatedText";
    for (int i = 0; i < args.Length; ++i)
    {
        switch (args[i].ToLowerInvariant())
        {
            case "/y":
            case "/Y":
            case "-y":
            case "-Y":
                yes2all = true;
                break;
            case "/q":
            case "/Q":
            case "-q":
            case "-Q":
                quiet = true;
                break;
            case "/n":
            case "-n":
                norandom = true;
                break;
            case "/f":
            case "/F":
            case "-f":
            case "-F":
                if (i + 1 < args.Length)
                {
                    if (Directory.Exists(args[++i]))
                        fname = Path.Combine(args[i], fname);
                    else
                        fname = args[i];
                }
                break;
        }
    }
    if (norandom)
        DsPdfWeb.Demos.Common.Util.NoRandom = true;
    if (!yes2all)
        quiet = false;
    if (quiet)
    {
        outMs = new MemoryStream();
        outTw = new StreamWriter(outMs);
        Console.SetOut(outTw);
    }

    var lk = Environment.GetEnvironmentVariable("DOCSOL_LKEY");
    if (!string.IsNullOrEmpty(lk))
        GcPdfDocument.SetLicenseKey(lk);

    var pdfName = Path.ChangeExtension(fname, ".pdf");
    var tiffName = Path.ChangeExtension(fname, ".tiff");
    var svgName = Path.ChangeExtension(fname, ".svg");

    if (!yes2all)
    {
        Console.WriteLine($"ENTER を押下して '{pdfName}' を作成するか、別のファイル名を入力してください：");
        var t = Console.ReadLine();
        if (!string.IsNullOrWhiteSpace(t))
            pdfName = t;
        pdfName = Path.GetFullPath(pdfName);
        if (File.Exists(pdfName))
        {
            Console.WriteLine($"'{pdfName}' は既に存在します。\n上書きしますか？(Y/n)");
            var yesno = Console.ReadLine();
            if (yesno?.ToLower() == "n")
            {
                Console.WriteLine("中断しました。");
                return 2;
            }
        }
    }
    if (!quiet)
        Console.WriteLine($"'{pdfName}' を生成しています...");
    var sample = new DsPdfWeb.Demos.Basics.RotatedText();
    using (var fs = new FileStream(pdfName, FileMode.Create))
    {
        sample.CreatePDF(fs);
        if (!quiet)
            Console.WriteLine($"'{pdfName}' は正常に作成されました。");

        if (!yes2all)
        {
            // TIFF のファイル名を取得します：
            Console.WriteLine($"Enter キーを押下して '{tiffName}' を作成するか、別のファイル名を入力してください：");
            var t = Console.ReadLine();
            if (!string.IsNullOrWhiteSpace(t))
                tiffName = t;
            tiffName = Path.GetFullPath(tiffName);
            if (File.Exists(tiffName))
            {
                Console.WriteLine($"'{tiffName}' は既に存在します。\n上書きしますか？(Y/n)");
                var yesno = Console.ReadLine();
                if (yesno?.ToLower() == "n")
                {
                    Console.WriteLine("中断しました。");
                    return 2;
                }
            }
            // SVG のファイル名を取得します：
            Console.WriteLine($"Enter キーを押下して '{svgName}' を作成するか、別のファイル名を入力してください：");
            t = Console.ReadLine();
            if (!string.IsNullOrWhiteSpace(t))
                svgName = t;
            svgName = Path.GetFullPath(svgName);
            if (File.Exists(svgName))
            {
                Console.WriteLine($"'{svgName}' は既に存在します。\n上書きしますか？(Y/n)");
                var yesno = Console.ReadLine();
                if (yesno?.ToLower() == "n")
                {
                    Console.WriteLine("中断しました。");
                    return 2;
                }
            }
        }
        // TIFF として保存します：
        if (!quiet)
            Console.WriteLine($"'{tiffName}' を生成しています...");
        fs.Seek(0, SeekOrigin.Begin);
        var pdf = new GcPdfDocument();
        // 注：DsPdf のライセンスがない場合、読み込まれるのは最大５ページまでです。
        pdf.Load(fs);
        // エクスポート時にデフォルトのレイヤー構成が使用されるように、デフォルトのビュー状態を作成します：
        var vs = new ViewState(pdf);
        pdf.SaveAsTiff(tiffName, vs);
        if (!quiet)
        {
            Console.WriteLine($"'{tiffName}' は正常に作成されました。");
            Console.WriteLine($"'{svgName}' を生成しています...");
        }
        // SVG として保存します：
        pdf.Pages[0].SaveAsSvg(svgName, vs);
        if (!quiet)
            Console.WriteLine($"'{svgName}' は正常に作成されました。");

        // ドキュメントのページは、サポートされている他の画像形式に保存することもできます。
        // 詳細については、GcPdfDocument.SaveAs<形式> または Page.SaveAs<形式> を参照してください。
    }
    // 生成された PDF ファイルを開くには、以下のコメントを解除してください：
    // var psi = new ProcessStartInfo() { FileName = pdfName, UseShellExecute = true };
    // Process.Start(psi);

    // 生成された TIFF ファイルを開くには、以下のコメントを解除してください：
    // var psiTiff = new ProcessStartInfo() { FileName = tiffName, UseShellExecute = true };
    // Process.Start(psiTiff);

    // 生成された SVG ファイルを開くには、以下のコメントを解除してください：
    // var psiSvg = new ProcessStartInfo() { FileName = svgName, UseShellExecute = true };
    // Process.Start(psiSvg);
}
catch (Exception ex)
{
    Console.Error.WriteLine($"エラー：{ex.Message}");
    if (ex.InnerException != null)
        Console.Error.WriteLine($"内部例外：{ex.InnerException.Message}");
    return 1;
}
finally
{
    if (!yes2all)
    {
        Console.WriteLine("終了するには何かキーを押下してください...");
        Console.ReadKey();
    }
    if (quiet)
    {
        outTw.Dispose();
        outMs.Dispose();
    }
}
return 0;
